@@ -318,7 +318,7 @@ nios2_control_register_arg_p (const char *str)
 	  || strprefix (str, "pteaddr")
 	  || strprefix (str, "tlbacc")
 	  || strprefix (str, "tlbmisc")
-	  || strprefix (str, "fstatus")
+	  || strprefix (str, "eccinj")
 	  || strprefix (str, "config")
 	  || strprefix (str, "mpubase")
 	  || strprefix (str, "mpuacc")
@@ -1258,7 +1258,8 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 	      fixup = fixup & 0xFFFF;
 	      break;
 	    case BFD_RELOC_NIOS2_HIADJ16:
-	      fixup = ((fixup >> 16) & 0xFFFF) + ((fixup >> 15) & 0x01);
+	      fixup = ((((fixup >> 16) & 0xFFFF) + ((fixup >> 15) & 0x01))
+		       & 0xFFFF);
 	      break;
 	    default:
 	      {
@@ -1693,6 +1694,28 @@ nios2_assemble_args_cs (nios2_insn_infoS *insn_info)
     }
 }
 
+static void
+nios2_assemble_args_ds (nios2_insn_infoS * insn_info)
+{
+  if (insn_info->insn_tokens[1] != NULL && insn_info->insn_tokens[2] != NULL)
+    {
+      struct nios2_reg *dst = nios2_reg_lookup (insn_info->insn_tokens[1]);
+      struct nios2_reg *src = nios2_reg_lookup (insn_info->insn_tokens[2]);
+
+      if (dst == NULL)
+	as_bad (_("unknown register %s"), insn_info->insn_tokens[1]);
+      else
+	SET_INSN_FIELD (RRD, insn_info->insn_code, dst->index);
+
+      if (src == NULL)
+	as_bad (_("unknown register %s"), insn_info->insn_tokens[2]);
+      else
+	SET_INSN_FIELD (RRS, insn_info->insn_code, src->index);
+
+      nios2_check_assembly (insn_info->insn_code, insn_info->insn_tokens[3]);
+    }
+}
+
 static void
 nios2_assemble_args_ldst (nios2_insn_infoS *insn_info)
 {
@@ -1850,6 +1873,8 @@ const nios2_arg_infoS nios2_arg_info_structs[] = {
   {"d,c,E", nios2_assemble_args_dc},
   {"c,s", nios2_assemble_args_cs},
   {"c,s,E", nios2_assemble_args_cs},
+  {"d,s", nios2_assemble_args_ds},
+  {"d,s,E", nios2_assemble_args_ds},
   {"l,d,s,t", nios2_assemble_args_ldst},
   {"l,d,s,t,E", nios2_assemble_args_ldst},
   {"d,s,j", nios2_assemble_args_dsj},
@@ -1926,15 +1951,17 @@ nios2_consume_arg (nios2_insn_infoS *insn, char *argstr, const char *parsestr)
       /* And whether we are using oci registers.  */
       if (!nios2_as_options.nobreak
 	  && (regno == 25 || strprefix (argstr, "bt")))
-	as_warn (_("The debugger will corrupt bt (r25). If you don't need to "
-		   "debug this\n"
-		   "code then use .set nobreak to turn off this warning."));
+	as_warn (_("The debugger will corrupt bt (r25).\n"
+		   "If you don't need to debug this "
+		   "code use .set nobreak to turn off this warning."));
 	
       if (!nios2_as_options.nobreak
-	  && (regno == 30 || strprefix (argstr, "ba")))
-	as_warn (_("The debugger will corrupt ba (r30). If you don't need to "
-		   "debug this\n"
-		   "code then use .set nobreak to turn off this warning."));
+	  && (regno == 30
+	      || strprefix (argstr, "ba")
+	      || strprefix (argstr, "sstatus")))
+	as_warn (_("The debugger will corrupt sstatus/ba (r30).\n"
+		   "If you don't need to debug this "
+		   "code use .set nobreak to turn off this warning."));
       break;
     case 'i':
     case 'u':
@@ -1957,14 +1984,16 @@ nios2_consume_arg (nios2_insn_infoS *insn, char *argstr, const char *parsestr)
       break;
     case 'm':
     case 'j':
-    case 'k':
     case 'l':
     case 'b':
       /* We can't have %hi, %lo or %hiadj here.  */
       if (*argstr == '%')
 	as_bad (_("badly formed expression near %s"), argstr);
       break;
+    case 'o':
+      break;
     default:
+      BAD_CASE (*parsestr);
       break;
     }
 
@@ -2052,7 +2081,11 @@ nios2_parse_args (nios2_insn_infoS *insn, char *argstr,
 
   parsed_args[i] = NULL;
 
-  if (*parsestr != '\0' && insn->insn_nios2_opcode->match != OP_MATCH_BREAK)
+  /* The argument to break and trap instructions is optional; complain
+     for other cases of missing arguments.  */
+  if (*parsestr != '\0'
+      && insn->insn_nios2_opcode->match != OP_MATCH_BREAK
+      && insn->insn_nios2_opcode->match != OP_MATCH_TRAP)
     as_bad (_("missing argument"));
 }
 
