@@ -126,7 +126,7 @@ ldfile_try_open_bfd (const char *attempt,
 {
   entry->the_bfd = bfd_openr (attempt, entry->target);
 
-  if (trace_file_tries)
+  if (verbose)
     {
       if (entry->the_bfd == NULL)
 	info_msg (_("attempt to open %s failed\n"), attempt);
@@ -467,7 +467,7 @@ try_open (const char *name, bfd_boolean *sysrooted)
   if (result != NULL)
     *sysrooted = is_sysrooted_pathname (name);
 
-  if (trace_file_tries)
+  if (verbose)
     {
       if (result == NULL)
 	info_msg (_("cannot find script file %s\n"), name);
@@ -604,6 +604,7 @@ ldfile_open_command_file_1 (const char *name, bfd_boolean default_only)
     {
       bfd_set_error (bfd_error_system_call);
       einfo (_("%P%F: cannot open linker script file %s: %E\n"), name);
+      return;
     }
 
   lex_push_file (ldlex_input_stack, name, sysrooted);
